<?php

/**
 * Plugin Name: WooCommerce Betatransfer Payment Gateway
 * Plugin URI: https://betatransfer.io/
 * Description: Payment gateway Betatransfer.
 * Version: 1.4.1
 * Author: Betatransfer
 * Author URI: https://betatransfer.io
 * Requires PHP: 7.2
 */

if (!defined('ABSPATH')) {
    exit;
}

add_filter('woocommerce_payment_gateways', 'woocommerce_add_betatransfer_gateway_class');

function woocommerce_add_betatransfer_gateway_class($gateways)
{
    $gateways[] = 'WC_Gateway_Betatransfer';
    return $gateways;
}

add_action('plugins_loaded', 'init_betatransfer_gateway_class');

function init_betatransfer_gateway_class()
{
    class WC_Gateway_Betatransfer extends WC_Payment_Gateway
    {
        const btUrl = 'https://merchant.betatransfer.io';

        /**
         * Token of Merchant.
         *
         * @var string
         */
        public $token;

        /**
         * Secret Key of Merchant.
         *
         * @var string
         */
        public $secret;

        /**
         * Payment system.
         *
         * @var string
         */
        public $payment_system;

        public function __construct()
        {
            $plugin_dir = basename(dirname(__FILE__));
            load_plugin_textdomain('betatransfer', false, $plugin_dir . '/languages');

            $this->id = 'betatransfer';
            $this->icon = apply_filters('woocommerce_betatransfer_icon', plugin_dir_url(__FILE__) . 'images/logo.png');
            $this->has_fields = false;
            $this->method_title = __('Betatransfer', 'betatransfer');
            $this->method_description = __('Payment via Betatransfer.', 'betatransfer');

            $this->supports = array(
                'products'
            );

            $this->init_form_fields();

            // Load the settings.
            $this->init_settings();
            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->token = trim($this->get_option('token'));
            $this->secret = trim($this->get_option('secret'));
            $this->payment_system = trim($this->get_option('payment_system'));

            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_api_wc_gateway_betatransfer', array($this, 'webhook'));

            if (!$this->is_valid_for_use()) {
                $this->enabled = 'no';
            }
        }

        public function init_form_fields()
        {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'betatransfer'),
                    'label' => __('Enable', 'betatransfer'),
                    'type' => 'checkbox',
                    'description' => '',
                    'default' => 'no'
                ),
                'title' => array(
                    'title' => __('Title', 'betatransfer'),
                    'type' => 'text',
                    'description' => __('A description of the payment method that the customer will see when placing an order.', 'betatransfer'),
                    'default' => __('Betatransfer', 'betatransfer'),
                    'desc_tip' => true,
                ),
                'description' => array(
                    'title' => __('Description', 'betatransfer'),
                    'type' => 'textarea',
                    'description' => __('A description that is displayed during the process of selecting a form of payment.', 'betatransfer'),
                    'default' => __('Pay through the electronic payment system of the Betatransfer', 'betatransfer'),
                ),
                'token' => array(
                    'title' => __('Public key', 'betatransfer'),
                    'type' => 'text',
                ),
                'secret' => array(
                    'title' => __('Secret key', 'betatransfer'),
                    'type' => 'text',
                ),
                'payment_system' => array(
                    'title' => __('Payment system', 'betatransfer'),
                    'type' => 'text',
                ),
            );
        }

        public function process_payment($order_id)
        {
            $order = wc_get_order($order_id);

            $params['amount'] = $order->get_total();
            $params['currency'] = get_woocommerce_currency();
            $params['orderId'] = $order_id;
            $params['locale'] = substr(get_locale(), 0, 2);
            $params['urlSuccess'] = esc_url_raw($this->get_return_url($order));
            $params['urlFail'] = esc_url_raw($order->get_cancel_order_url());
            $params['urlResult'] = esc_url_raw(add_query_arg('wc-api', 'WC_Gateway_Betatransfer', home_url('/')));
            $params['payerId'] = hash('crc32', $order->get_customer_id());
            $params['payerPhone'] = $order->get_billing_phone();
            $params['payerEmail'] = $order->get_billing_email();
            $params['payer_firstname'] = $order->get_billing_first_name();
            $params['payer_lastname'] = $order->get_billing_last_name();
            $params['fullCallback'] = 1;
            $params['paymentSystem'] = $this->payment_system;
            $params['sign'] = $this->generate_sign(array_filter($params), $this->secret);

            $response = wp_remote_post(self::btUrl . '/api/payment?token=' . $this->token, array(
                'headers' => array('Content-Type' => 'application/x-www-form-urlencoded'),
                'body' => array_filter($params)
            ));

            $response_status_code = wp_remote_retrieve_response_code($response);

            if (200 === $response_status_code) {
                $body = json_decode(wp_remote_retrieve_body($response), true);

                if (!empty($body['urlPayment'])) {
                    return array(
                        'result' => 'success',
                        'redirect' => $body['urlPayment'],
                    );
                } else {
                    wc_add_notice(__('Try again.', 'betatransfer'), 'error');
                    return;
                }
            } elseif (403 === $response_status_code) {
                wc_add_notice(__('Problem connecting to the gateway.', 'betatransfer'), 'error');
                return;
            } elseif (422 === $response_status_code) {
                wc_add_notice(__('Problem when processing a request.', 'betatransfer'), 'error');
                return;
            } else {
                wc_add_notice(__('Problem when trying to connect.', 'betatransfer'), 'error');
                return;
            }
        }

        public function webhook()
        {
            $order = wc_get_order($_POST['orderId']);
            $sign = $this->generate_callback_sign($_POST, $this->secret);

            if ($_POST['sign'] == $sign) {
                if (in_array($_POST['status'], array('success', 'partial_payment'))) {
                    $order->payment_complete();
                    $order->add_order_note(__('The payment was successfully paid through the Betatransfer', 'betatransfer'));
                } elseif ($_POST['status'] == 'cancel') {
                    $order->update_status('cancelled', __('Payment canceled', 'betatransfer'));
                    $order->add_order_note(__('Payment canceled', 'betatransfer'));
                } else {
                    $order->update_status('failed', __('Payment not paid', 'betatransfer'));
                    $order->add_order_note(__('Payment not paid', 'betatransfer'));
                }

                echo 'OK';
                header('HTTP/1.1 200 OK');
            } else {
                wp_redirect($order->get_cancel_order_url());
            }

            die('Bad Request.');
        }

        private function is_valid_for_use()
        {
            return in_array(get_woocommerce_currency(), array('EUR', 'RUB', 'UAH', 'USD'));
        }

        private function generate_sign($data, $secret)
        {
            if (!empty($data['sign'])) {
                unset($data['sign']);
            }

            array_push($data, $secret);
            return md5(implode('', $data));
        }

        private function generate_callback_sign($data, $secret)
        {
            $data = array_filter($data, function($key) {
                return in_array($key, array('amount', 'orderId'));
            }, ARRAY_FILTER_USE_KEY);

            ksort($data);

            return $this->generate_sign($data, $secret);
        }
    }
}